<?php

/**
 * @file
 * Contains \Drupal\lingotek\Remote\LingotekApi.
 */

namespace Drupal\lingotek\Remote;

use Drupal\lingotek\Exception\LingotekApiException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/*
 * a simple connector to the Lingotek Translation API
 */

class LingotekApi implements LingotekApiInterface {

  /**
   * The HTTP client to interact with the Lingotek service.
   *
   * @var \Drupal\lingotek\Remote\LingotekHttpInterface
   */
  protected $lingotekClient;

  /**
   * Constructs a LingotekApi object.
   */
  public function __construct(LingotekHttpInterface $client) {
    $this->lingotekClient = $client;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('lingotek.http_client')
    );
  }

  public function getAccountInfo() {
    try {
      $access_token = $this->lingotekClient->getCurrentToken();
      $account_info = $this->lingotekClient->get('/auth/oauth2/access_token_info?access_token=' . $access_token);
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get account info: ' . $e->getMessage());
    }
    return $account_info;
  }

  public function addDocument($args) {
    try {
      $response = $this->lingotekClient->post('/api/document', $args, TRUE);
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to add document: ' . $e->getMessage());
    }
    if ($response->getStatusCode() == '202') {
      $data = json_decode($response->getBody(), TRUE);
      if (!empty($data['properties']['id'])) {
        return $data['properties']['id'];
      }
    }
    // TODO: log warning
    return FALSE;
  }

  public function patchDocument($id, $args) {
    try {
      $response = $this->lingotekClient->patch('/api/document/' . $id, $args, TRUE);
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to patch (update) document: ' . $e->getMessage());
    }
    return $response;
  }

  public function deleteDocument($id) {
    try {
      $response = $this->lingotekClient->delete('/api/document' . '/' . $id);
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to delete document: ' . $e->getMessage());
    }
    return $response;
  }

  public function getDocument($id) {
    try {
      $response = $this->lingotekClient->get('/api/document', array('doc_id' => $id));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get document: ' . $e->getMessage());
    }
    return $response;
  }

  public function documentExists($id) {
    // TODO
    throw new Exception('Not implemented');
  }

  public function getDocumentStatus($id) {
    try {
      $response = $this->lingotekClient->get('/api/document/' . $id . '/status');
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get document status: ' . $e->getMessage());
    }
    return $response;
  }

  public function addTranslation($id, $locale) {
    try {
      $response = $this->lingotekClient->post('/api/document/' . $id . '/translation', array('locale_code' => $locale));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to add translation: ' . $e->getMessage());
    }
    return $response;
  }

  public function getTranslation($id, $locale) {
    try {
      $response = $this->lingotekClient->get('/api/document/' . $id . '/content', array('locale_code' => $locale));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to add translation: ' . $e->getMessage());
    }
    return $response;
  }

  public function deleteTranslation($id, $locale) {
    try {
      $response = $this->lingotekClient->delete('/api/document/' . $id . '/translation', array('locale_code' => $locale));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to add translation: ' . $e->getMessage());
    }
    return $response;
  }

  public function getCommunities() {
    try {
      $response = $this->lingotekClient->get('/api/community?limit=50');
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get communities: ' . $e->getMessage());
    }
    return $this->formatResponse($response);
  }

  public function getProject($project_id) {
    try {
      $response = $this->lingotekClient->get('/api/project/' . $project_id); 
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get project: ' . $e->getMessage());
    }
    return $response->json();
  }

  public function getProjects($community_id) {
    try {
      $response = $this->lingotekClient->get('/api/project', array('community_id' => $community_id, 'limit' => 100));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get projects: ' . $e->getMessage());
    }
    return $this->formatResponse($response);
  }

  public function setProjectCallBackUrl($project_id, $args) {
    try {
      $response = $this->lingotekClient->patch('/api/project/' . $project_id, $args);
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to patch project: ' . $e->getMessage());
    }
    return $response;
  }


  public function getVaults($community_id) {
    try {
      $response = $this->lingotekClient->get('/api/vault', array('community_id' => $community_id));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get vaults: ' . $e->getMessage());
    }
    return $this->formatResponse($response);
  }

  public function getWorkflows($community_id) {
    try {
      $response = $this->lingotekClient->get('/api/workflow', array('community_id' => $community_id));
    }
    catch (\Exception $e) {
      throw new LingotekApiException('Failed to get workflows: ' . $e->getMessage());
    }
    return $this->formatResponse($response);
  }

  protected function formatResponse($response) {
    $formatted_response = array();
    $json_response = json_decode($response->getBody(), TRUE);
    if (!empty($json_response['entities'])) {
      foreach ($json_response['entities'] as $entity) {
        if (!empty($entity['properties']['id']) && !empty($entity['properties']['title'])) {
          $formatted_response[$entity['properties']['id']] = $entity['properties']['title'];
        }
      }
    }
    return $formatted_response;
  }

}
